/* ====================================================================
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is SATC.
 *
 * The Initial Developer of the Original Code is Bogdan Mitu
 *
 * Copyright (C) 2001-2002 Bogdan Mitu
 *
 * ====================================================================
 */

header {
package satc.lang; 
}

class ELexer extends Lexer;

options {
    k = 3;                           
    testLiterals = false;
    charVocabulary = '\3'..'\377';
    importVocab = MLC;
    exportVocab=E; 
    filter = true;     
}

tokens {
// predefined types
    "int"       ;
    "uint"      ;
    "bit"       ;
    "byte"      ;
    "time"      ;
    "bool"      ;
    "string"    ;
    "locker"    ;
    "file"      ;
// constants
    "TRUE"      ;
    "FALSE"     ;
    "NULL"      ;
    "UNDEF"     ;
    "MAX_INT"   ;
    "MAX_UINT"  ;
    "MIN_INT"   ;
// user defined types (statements)
    "type"      ;
    "bits"      ;
    "bytes"     ;
// type conversions (actions)
    "as_a"      ;
// variables (actions)
    "var"       ;
// operators
    "not"       ;
    "and"       ;
    "or"        ;
    "in"        ;
    "is"        ;
    "a"         ;
// loops
    "while"     ;
    "do"        ;
    "for"       ;
    "from"      ;
    "down"      ;
    "to"        ;
    "step"      ;
    "each"      ;
    "using"     ;
    "index"     ;
    "matching"  ;
    "line"      ;
    "in_file"   ;
    "break"     ;
    "continue"  ;
// flow control
    "if"        ;
    "then"      ;
    "else"      ;
    "case"      ;
    "default"   ;
    "all"       ;
    "of"        ;
// structs & units
    "struct"    ;
    "unit"      ;
    "like"      ;
    "extend"    ;
    "instance"  ;
    "when"      ;
// predefined methods
    "init"  ;
    "pre_generate";
    "post_generate";
    "run"       ;
    "extract"   ;
    "quit"  ;
    "finalize";
    "copy"  ;
    "do_print";
    "print_line";
    "do_pack"   ;
    "do_unpack";
    "hdl_path";
    "get_unit";
    "get_enclosing_unit";
    "try_enclosing_unit";
// methods & TCMs
    "first"     ;
    "also"      ;
    "only"      ;
    "sync"      ;
    "wait"      ;
    "until"     ;
    "result"    ;
    "return"    ;
    "start"     ;
// checking
    "check"     ;
    "that"      ;
    "dut_error" ;
    "expect"    ;
    "set_check" ;
    "ERROR"     ;
    "ERROR_BREAK_RUN";
    "ERROR_AUTOMATIC";
    "ERROR_CONTINUE";
    "WARNING"   ;
    "IGNORE"    ;
// coverage
    "cover"     ;
    "item"      ;
    "cross"     ;
    "transition";

    "no_collect";
    "at_least"  ;
    "global"    ;
    "text"      ;
    "weight"    ;

    "radix"     ;
    "DEC"       ;
    "BIN"       ;
    "HEX"       ;

    "ignore"    ;
    "illegal"   ;
    "ranges"    ;
    "range"     ;

    "mode"      ;
    "normal"    ;
    "count_only";
// packing & simulator interface
    "pack"      ;
    "unpack"    ;

    "packing"   ;
    "high"      ;
    "high_big_endian"   ;
    "low"   ;
    "low_big_endian"    ;
    "network"   ;

    "swap"      ;

    "code"      ;
    "function"  ;
// Verilog
    "verilog"   ;

    "variable"  ;
    "net"       ;
    "wire"      ;
    "forcible"  ;
    "drive"     ;
    "hold"      ;
    "strobe"    ;

    "task"      ;
// VHDL
    "driver"    ;
    "disconnect";
    "value"     ;
    "delay"     ;
    "INERTIAL"  ;
    "TRANSPORT" ;
    "initial"   ;

    "interface" ;
    "library"   ;
    "package"   ;
    "alias"     ;
    "declarative_item";

    "procedure" ;
// force & release
    "force"     ;
    "release"   ;
//events & temporal expressions
    "event"     ;
    "emit"      ;
    "on"        ;

    "sys"   ;
    "any"   ;
    "tick_start";
    "tick_end"  ;
    "new_time"  ;

    "session"   ;
    "start_of_test" ;
    "end_of_test"   ;

    "change"    ;
    "rise"      ;
    "fall"      ;
    "sim"       ;
    "true"      ;
    "cycle"     ;
    "fail"      ;
    "eventually";
    "detach"    ;
    "exec"      ;
    "consume"   ;
// generation & constraints
    "gen"       ;
    "keep"      ;
    "keeping"   ;
    "soft"      ;
    "prev"      ;
    "before"    ;
    "select"    ;
    "is_all_iterations";
    "reset_soft";
// lists & list pseudo-methods - direct modification of a list
    "list"      ;
    "key"       ;
    "add"       ;
//  "add0"      ;
    "clear" ;       
    "delete"    ;
    "fast_delete";
    "insert"    ;
    "pop"       ;
    "push"      ;
//  "push0"     ;
    "resize"    ;
// lists & list pseudo-methods - queries & general operations on a list
    "all_indicies";
    "apply"     ;
    "count"     ;
    "exists"    ;
    "first_index";
    "get_indicies";
    "has"       ;
    "is_a_permutation";
    "is_empty"  ;
    "last"      ;
    "last_index";
    "max"       ;
    "max_index" ;
    "max_value" ;
    "min"       ;
    "min_index" ;
    "min_value" ;
    "reverse"   ;
    "size"      ;
    "sort"      ;
    "sort_by_field";
    "split"     ;
    "top"       ;
//  "top0"      ;
    "unique"    ;
// lists & list pseudo-methods - math & logic operations on a list
    "and_all"   ;
    "average"   ;
    "or_all"    ;
    "product"   ;
    "sum"       ;
//  "crc_8"     ;
//  "crc_32"    ;
//  "crc_32_flip";
// lists & list pseudo-methods - specifics for keyed lists
    "key_index" ;
    "key_exists";   
// actions (predefined routines)
    "out"       ;
    "outf"      ;

    "abs"       ;
    "odd"       ;
    "even"      ;
    "ilog"      ;
    "ipow"      ;
    "isqrt"     ;
    "div_round_up";

    "bit_wise_and";
    "bit_wise_or";
    "bit_wise_xor";
    "bit_wise_nand";
    "bit_wise_nor";
    "bit_wise_xnor";

    "deep_copy" ;
    "deep_compare";
    "deep_compare_physical";

    "append"    ;
    "appendf"   ;
    "bin"       ;
    "dec"       ;
    "hex"       ;
    "quote"     ;
    "str_chop"  ;
    "str_empty" ;
    "str_exactly";
    "str_expand_dots";
    "str_insensitive";
    "str_join"  ;
    "str_len"   ;
    "str_lower" ;
    "str_match" ;
    "str_pad"   ;
    "str_replace";
    "str_split" ;
    "str_split_all";
    "str_sub"   ;
    "str_upper" ;
    "to_string" ;

    "set_config";
    "get_config";
    "write_config"; 
    "read_config";
    "set_keep"  ;
    "get_keep"  ;
    "with"      ;

    "specman"   ;
    "spawn"     ;
    "spawn_check";

    "system"    ;
    "output_from";
    "output_from_check";
    "get_symbol";
    "date_time" ;
// miscellaneous
//  "#ifdef"    ;
//  "#ifndef"   ;
    "import"    ;
    "as"        ;
    "computed"  ;

    "define"    ;

    "lock"      ;   
    "unlock"    ;

    "state"     ;
    "machine"   ;
    "files"     ;

    "open"      ;
    "read"      ;
    "write"     ;
    "read_lob"  ;
    "write_lob" ;
    "close"     ;
    "flush"     ;
    "add_file_type";

}

{
public ELexer() {
    this((LexerSharedInputState) null); 
}
}

ID  options{ testLiterals = true;}
        : ('a'..'z'|'A'..'Z') ('a'..'z'|'A'..'Z'|'0'..'9'|'_')* ;

TAB     : '\t'  ;

BLANK   : ' '   ;

NEWLINE : ('\r' | '\n' | "\r\n") { newline();}   ;

LBRACK          :   '['     ;
RBRACK          :   ']'     ;
DOT             :   '.'     ;
SLICE           :   ".."    ;
COLON           :   ':'     ;
LPAREN          :   '('     ;
RPAREN          :   ')'     ;
TILDE           :   '~'     ;
EXCL            :   '!'     ;
LCURLY          :   '{'     ;
RCURLY          :   '}'     ;
SEMI            :   ';'     ;
COMMA           :   ','     ;
PLUS            :   '+'     ;
MINUS           :   '-'     ;
STAR            :   '*'     ;
DIV             :   '/'     ;
MOD             :   '%'     ;
SHL             :   "<<"    ;
SHR             :   ">>"    ;
GT              :   ">"     ;
LT              :   "<"     ;
LE              :   "<="    ;
GE              :   ">="    ;
//EQUAL         :   '='     ;
EQEQ            :   "=="    ;
NEQ             :   "!="    ;
EQEQEQ          :   "==="   ;
NEQEQ           :   "!=="   ;
NMATCH          :   "!~"    ;
BXOR            :   '^'     ;
BOR             :   '|'     ;
BAND            :   '&'     ;
LOR             :   "||"    ;
LAND            :   "&&"    ;
SPD1            :   "=>"    ;
QUESTION        :   '?'     ;
AT              :   '@'     ;
ACC             :   '\''    ;
ACC2            :   '`'    ;

/*SHARP           : '#'     ;
RARROW          :   "->"    ;

SPD2            :   "*>"    ;


NOR             :   "~|"    ;
NAND            :   "~&"    ;
NXOR            :   "~^"    ;


*/

// escape sequence -- note that this is protected; it can only be called
//   from another lexer rule -- it will not ever directly return a token to
//   the parser
protected ESC
        :   '\\'
        (   'n'
        |   'r'
        |   't'
        |   'f'
        |   '"'
        |   '\\'
        )
        ;

// string literals
STRING_LITERAL
        :   '"' (ESC|~('"'|'\\'))* '"'
        ; 

protected DEC
        : ('0'..'9')('0'..'9'|'_')*
        ;
protected BIN
        : ('0'..'1')('0'..'1'|'_')*
        ;
protected OCT
        : ('0'..'7')('0'..'7'|'_')*
        ;
protected HEX
        : ('0'..'9'|'a'..'f'|'A'..'F')('0'..'9'|'a'..'f'|'A'..'F'|'_')*
        ;
protected D_INT
        : DEC('K'|'M')?
        ;
B_INT   
        : ("0b" | "0B")BIN
        ;
O_INT   
        : ("0o" | "0O")OCT
        ;
H_INT   
        : ("0h" | "0H" | "0x" | "0X")HEX
        ;
protected SIZED_D_INT
        : ('0'..'9')+("'d" | "'D")DEC
        ;
protected SIZED_B_INT   
        : ('0'..'9')+("'b" | "'B")BIN
        ;
protected SIZED_O_INT
        : ('0'..'9')+("'o" | "'O")OCT
        ;
protected SIZED_H_INT
        : ('0'..'9')+("'h" | "'H" | "'x" | "'X")HEX
        ;

D_INT_OR_SIZED 
        : (('0'..'9')+ ("'d" | "'D")) => SIZED_D_INT 
        | (('0'..'9')+ ("'b" | "'B")) => SIZED_B_INT
        | (('0'..'9')+ ("'h" | "'H")) => SIZED_H_INT
        | (('0'..'9')+ ("'o" | "'O")) => SIZED_O_INT
        | D_INT
        ;

CHAR_LITERAL    
        : '0' ('c' | 'C') "\""."\""
        ;

SL_COMMENT 
        :   ("//" (~('\r' | '\n'))*) | ("--" (~('\r' | '\n'))*)
        ;

DIRECTIVE
        :   '#' 
            (   ("define")              => "define" 
            |   ("ifdef")               => "ifdef" 
            |   ("ifndef")              => "ifndef" 
            )
        ;    
        
SYSTASK 
        :   '$'('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'0'..'9'|'_')*
        ;
        
MLC_OPEN
        :   "/*"
        ;

