/* ====================================================================
 *
 * This grammar is an adaptation of a public domain Verilog grammar 
 * written by Steve Eckmann. 
 *
 * ====================================================================
 */

header {
package satc.lang; 
}

class VerilogLexer extends Lexer;

options {
    k = 3;                           
    testLiterals = false;
    charVocabulary = '\3'..'\377';
    importVocab = MLC;
    exportVocab=VerilogLexer;
//    filter = true;     
}

tokens {
//    INVALID="INVALID";
    "module"        ;
    "macromodule"   ;
    "begin"         ;
    "end"           ;
    "initial"       ;
    "always"        ;
    "endmodule"     ;
    "case"          ;
    "casex"         ;
    "casez"         ;
    "endcase"       ;
    "default"       ;
    "if"            ;
    "then"          ;
    "else"          ;
    "forever"       ;
    "repeat"        ;
    "while"         ; 
    "for"           ; 
    "wait"          ; 
    "posedge"       ;
    "negedge"       ;
    "parameter"     ;
    "input"         ;
    "output"        ;
    "inout"         ;
    "trireg"        ;
    "wire"          ;
    "tri"           ;
    "wand"          ;
    "triand"        ;
    "wor"           ;
    "trior"         ;
    "scalared"      ;
    "vectored"      ;
    "reg"           ;
    "time"          ;
    "integer"       ;
    "real"          ;
    "event"         ;
    "assign"        ;
    "defparam"      ;
    "small"         ;
    "medium"        ;
    "large"         ;
    "and"           ;
    "nand"          ;
    "or"            ;
    "nor"           ;
    "xor"           ;
    "xnor"          ;
    "buf"           ;
    "not"           ;

    "supply0"       ;
    "strong0"       ;
    "pull0"         ;
    "weak0"         ;
    "highz0"        ;
    "supply1"       ;
    "strong1"       ;
    "pull1"         ;
    "weak1"         ;
    "highz1"        ;
    "buffif0"       ;
    "buffif1"       ;
    "notif0"        ;
    "notif1"        ;
    "tranif0"       ;
    "rtranif0"      ;
    "tranif1"       ;
    "rtranif1"      ;

/*    STRONG_0="strong0";
    STRONG_1="strong1";
//    "strong"        ;
    PULL_0="pull0";
    PULL_1="pull1";
//    "pull"          ;
    WEAK_0="weak0"  ;
    WEAK_1="weak1"  ;
//    "weak"          ;
    HIGHZ_0="highz0";
    HIGHZ_1="highz1";
//    "highz"         ;
    BUFFIF_0="buffif0";
    BUFFIF_1="buffif1";
//    "bufif"         ;
    NOTIF_0="notif0";
    NOTIF_1="notif1";
//    "notif"         ;
    TRANIF_0="tranif0";
    TRANIF_1="tranif1";
//    "tranif"        ;
    RTRANIF_0="rtranif0";
    RTRANIF_1="rtranif1";
//   "rtranif"       ;
*/
    "pulldown"      ;
    "pullup"        ;
    "nmos"          ;
    "rnmos"         ;
    "pmos"          ;
    "rpmos"         ;
    "cmos"          ;
    "rcmos"         ;
    "tran"          ;
    "rtran"         ;
    "disable"       ;
    "deassign"      ;
    "force"         ;   
    "release"       ;  
    "fork"          ;
    "join"          ;
    "function"      ;
    "endfunction"   ;   
    "task"          ;   
    "endtask"       ;
    "table"         ;
    "endtable"      ;
    "primitive"     ;
    "endprimitive"  ;
    "edge"          ; 
    "specparam"     ;
    "specify"       ; 
    "endspecify"    ;
}

{
public VerilogLexer() {
    this((LexerSharedInputState) null); 
}
}

TAB     : '\t'      ;

BLANK   : ' '      ; 

NEWLINE : ('\r' | '\n' | "\r\n") { newline();}      ;

SL_COMMENT 
    : "//" (~('\r' | '\n'))*  ;

MLC_OPEN
	:	"/*"
		
	;

////////////////////////////////////////////////
// Operators
////////////////////////////////////////////////
AT          : "@"   ;
COLON       : ":"   ;
COMMA       : ","   ;
DOT         : "."   ;
ASSIGN      : "="   ;
MINUS       : "-"   ;
LBRACK      : "["   ;
RBRACK      : "]"   ;
LCURLY      : "{"   ;
RCURLY      : "}"   ;
LPAREN      : "("   ;
RPAREN      : ")"   ;
POUND       : "#"   ;
QUESTION    : "?"   ;
SEMI        : ";"   ;
PLUS        : "+"   ;
LNOT        : "!"   ;
BNOT        : "~"   ;
BAND        : "&"   ;
RNAND       : "~&"  ;
BOR         : "|"   ;
RNOR        : "~|"  ;
BXOR        : "^"   ;
RXNOR       : "~^" | "^~" ;
STAR        : "*"   ;
DIV         : "/"   ;
MOD         : "%"   ;
EQUAL       : "=="  ;
NOT_EQ      : "!="  ;
NOT_EQ_CASE : "!==" ;
EQ_CASE     : "===" ;
LAND        : "&&"  ;
LOR         : "||"  ;
LT          : "<"   ;
LE          : "<="  ;
GT          : ">"   ;
GE          : ">="  ;
SR          : ">>"  ;
SL          : "<<"  ;
TRIGGER     : "->"  ;
PPATH       : "=>"  ;
FPATH       : "*>"  ;

    // an identifier.  Note that testLiterals is set to true!  This means
    // that after we match the rule, we look in the Literals table to see
    // if it's a literal or really an identifer.
IDENTIFIER
      options {testLiterals=true;}
    :
        ('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'_'|'$'|'0'..'9')*
        ;

ESCAPED_IDENTIFIER :
        '\\'! (~ '\040')+ ('\040'|'\t'|'\n')!
        ;

SYSTEM_TASK_NAME :
        '$' IDENTIFIER
        ;

    // string literals
STRING :
        '"' (~('"'|'\n'))* '"'
        ;

    // "compiler" define/macro.
DEFINE
      options {testLiterals=true;}
    :   '`' ('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'_'|'$'|'0'..'9')*
    ;

   // a dummy rule to force vocabulary to be all characters (except special
   //   ones that ANTLR uses internally (0 to 2)
protected
VOCAB :
        '\3'..'\177'
        ;

   // a numeric literal
NUMBER :
    ( (SIZE)? BASE SIZED_DIGIT ) => SIZED_NUMBER |
    UNSIZED_NUMBER
    ;

protected
SIZED_NUMBER :
    (SIZE)? BASE SIZED_DIGIT (SIZED_DIGIT | '_')*
    ;

protected
SIZE :
    (DIGIT)+
    ;

protected
BASE :
    '\'' ( 'd' | 'D' | 'h' | 'H' | 'o' | 'O' | 'b' | 'B' )
    ;

protected
SIZED_DIGIT :
    DIGIT | HEXDIGIT | 'x' | 'X' | 'z' | 'Z' | '?'
    ;

protected
UNSIZED_NUMBER :
    DIGIT (DIGIT | '_')* ( '.' (DIGIT | '_')* )? (EXPONENT)?
        ;

protected
DIGIT :
        ('0'..'9')
        ;

protected
HEXDIGIT :
        ('A'..'F'|'a'..'f')
        ;

protected
EXPONENT :
        ('e'|'E') ('+'|'-')? ('0'..'9')+
        ;



      
