/* ======================================================================
 *
 * This grammar is based on the ANTLR grammar from ANTLR distribution.
 *
 * ======================================================================
 */

header {
package satc.lang;
}

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/RIGHTS.html
 *
 * $Id: ANTLRLexer.g,v 1.1 2002/06/05 05:38:02 bogdan Exp $
 */

class ANTLRLexer extends Lexer;
options {
    k=2;
    importVocab = ANTLROptionLexer;
    exportVocab=ANTLRLexer;
    testLiterals=false;
    charVocabulary = '\3'..'\176'; 
    filter = true;
    defaultErrorHandler=false;
}

tokens {
    "options"       ;
    "tokens"        ;
    "header"        ;
    "lexclass"      ;
    "class"         ;
    "extends"       ;
    "Lexer"         ;
    "TreeParser"    ;
    "Parser"        ;
    "protected"     ;
    "public"        ;
    "private"       ;
    "returns"       ;
    "throws"        ;
    "exception"     ;
    "catch"         ;
//    "charVocabulary";
//    "importVocab"   ;
//    "exportVocab"   ;
//    "testLiterals"  ;
//    "warnWhenFollowAmbig" ; 
//    "filter"        ;
//    "true"          ;
//    "false"         ;
    
}

{
    public ANTLRLexer() {
        this((LexerSharedInputState) null); 
    }
}

SL_COMMENT : "//" (~('\r' | '\n'))*  ;  

NEWLINE : ('\r' | '\n' | "\r\n") (' '|'\t')* { newline();} ;

TAB     : '\t' ;

BLANK   : ' ' ;

MLC_OPEN : "/*" ;

TREE_BEGIN : "#(" ;

LPAREN      :   '('     ;
RPAREN      :   ')'     ;
LBRACK      :   '['     ;
RBRACK      :   ']'     ;
LCURLY      :   (ACTION) => ACTION   {$setType(ACTION);}
            |   '{' 
            ;
protected
ACTION      :   '{' (~('}' | '{' | '\'') | ('\'' . '\'') | ACTION)* '}'   ;

RCURLY      :   '}'     ;
COLON       :   ':'     ;
COMMA       :   ','     ;
STAR        :   '*'     ;
PLUS        :   '+'     ;
IMPLIES     :   "=>"    ;
SEMI        :   ';'     ;
CARET       :   '^'     ;
OR          :   '|'     ;
QUESTION    :   '?'     ;
DOT         :   '.'     ;
EQUAL       :   '='     ;
EXCL        :   '!'     ;
TILDE       :   '~'     ;
DIV         :   '/'     ;
RANGE       :   ".."    ;
MINUS       :   '-'     ;
MOD         :   '%'     ;
GT          :   ">"     ;
LT          :   ('<' MARKER (ARG)? '>') => '<' MARKER (ARG)? '>'   {$setType(MARKER);}   
            |   '<'
            ;

protected
MARKER      :   ('a'..'z' | 'A'..'Z' | '_' | '0'..'9' | '-' | '.' | '$')+   ;

protected
ARG	:	'(' XID ')'   ;
protected
XID  :   ('a'..'z' | 'A'..'Z' | '_' | '0'..'9' | '-' | '.' | '$')+   ;


BAND        :   '&'     ;
AT          :   '@'     ;
SHARP       :   '#'     ;
GRAVE       :   '`'     ;
//ACUTE     :   '\''    ;
BACK        :   '\\'    ;
DOLLAR      :   '$'     ;

ID  options{ testLiterals = true; }
        : ('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'0'..'9'|'_')* ;

CHAR_LITERAL
    :   '\'' (ESC|~'\'') '\''
    ;

STRING_LITERAL
    :   '"' (ESC|~'"')* '"'
    ;

protected
ESC :   '\\'
        (   'n'
        |   'r'
        |   't'
        |   'b'
        |   'f'
        |   'w'
        |   'a'
        |   '"'
        |   '\''
        |   '\\'
        |   ('0'..'3')
            (
                options {
                    warnWhenFollowAmbig = false;
                }
            :
    ('0'..'9')
                (   
                    options {
                        warnWhenFollowAmbig = false;
                    }
                :
    '0'..'9'
                )?
            )?
        |   ('4'..'7')
            (
                options {
                    warnWhenFollowAmbig = false;
                }
            :
    ('0'..'9')
            )?
        |   'u' XDIGIT XDIGIT XDIGIT XDIGIT
        )
    ;

protected
DIGIT
    :   '0'..'9'
    ;

protected
XDIGIT :
        '0' .. '9'
    |   'a' .. 'f'
    |   'A' .. 'F'
    ;

INT :   ('0'..'9')+
    ;
    