/* ====================================================================
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is SATC.
 *
 * The Initial Developer of the Original Code is Bogdan Mitu
 *
 * Copyright (C) 2001-2002 Bogdan Mitu
 *
 * ====================================================================
 */

header {
package satc.lang; 
}

class VHDLLexer extends Lexer;

options {
    k = 3;                           
    testLiterals = false;
    charVocabulary = '\3'..'\377';
    importVocab = MLC;
    exportVocab=VHDL;
    caseSensitive = false;
    caseSensitiveLiterals = false;
    filter = true;     
}

tokens {
//    INVALID="INVALID";
    "abs"           ;
    "access"        ;
    "after"         ;
    "alias"         ;
    "all"           ;
    "and"           ;
    "architecture"  ;
    "array"         ;
    "assert"        ;
    "attribute"     ;
    "begin"         ;
    "block"         ;
    "body"          ;
    "buffer"        ;
    "bus"           ;
    "case"          ;
    "component"     ;
    "configuration" ;
    "constant"      ;
    "disconnect"    ;
    "downto"        ;
    "else"          ;
    "elsif"         ;
    "end"           ;
    "entity"        ;
    "exit"          ;
    "file"          ;
    "for"           ;
    "function"      ;
    "generate"      ;
    "generic"       ;
    "group"         ;
    "guarded"       ;
    "if"            ;
    "impure"        ;
    "inout"         ;
    "in"            ;
    "inertial"      ;
    "is"            ;
    "label"         ;
    "library"       ;
    "linkage"       ;
    "literal"       ;
    "loop"          ;
    "map"           ;
    "mod"           ;
    "nand"          ;
    "new"           ;
    "next"          ;
    "nor"           ;
    "not"           ;
    "null"          ;
    "of"            ;
    "on"            ;
    "open"          ;
    "or"            ;
    "others"        ;
    "out"           ;
    "package"       ;
    "port"          ;
    "postponed"     ;
    "procedure"     ;
    "process"       ;
    "pure"          ;
    "range"         ;
    "record"        ;
    "register"      ;
    "reject"        ;
    "rem"           ;
    "report"        ;
    "return"        ;
    "rol"           ;
    "ror"           ;
    "select"        ;
    "severity"      ;
    "sla"           ;
    "sll"           ;
    "sra"           ;
    "srl"           ;
    "shared"        ;
    "signal"        ;
    "subtype"       ;
    "then"          ;
    "to"            ;
    "transport"     ;
    "type"          ;
    "units"         ;
    "until"         ;
    "unaffected"    ;
    "use"           ;
    "variable"      ;
    "wait"          ;
    "when"          ;
    "while"         ;
    "with"          ;
    "xor"           ;
    "xnor"          ;
/*
    "active"        ;
    "ascending"     ;
    "base"          ;
    "delayed"       ;
    "driving"       ;
    "driving_value" ;
    "event"         ;
    "high"          ;
    "image"         ;
    "instance"      ;
    "last_active"   ;
    "last_event"    ;
    "last_value"    ;
    "leftof"        ;
    "left"          ;
    "length"        ;
    "low"           ;
    "path"          ;
    "pos"           ;
//  "range"         ;
    "pred"          ;
    "rightof"       ;
    "right"         ;
    "reverse_range" ;
    "simple_name"   ;
    "stable"        ;
    "succ"          ;
    "transaction"   ;
    "quiet"         ;
    "value"         ;
    "val"           ;
*/
}

{
public VHDLLexer() {
    this((LexerSharedInputState) null); 
}
}

ATTRIB  : '\'' ('a'..'z'|'A'..'Z') ('a'..'z'|'A'..'Z'|'_'|'0'..'9')* ;

TAB     : '\t' ;

BLANK   : ' ' ;

NEWLINE : ('\r' | '\n' | "\r\n") { newline();} ;

SL_COMMENT 
    : "--" (~('\r' | '\n'))*  ;

MLC_OPEN
        : "/*"
        ;

    
  ////////////////////////////////////////////////
  // Operators
AT      : "@"   ;
COLON       : ":"   ;
COMMA       : ","   ;
DOT     : "."   ;
ASSIGN      : "="   ;
MINUS       : "-"   ;
LBRACK      : "["   ;
RBRACK      : "]"   ;
LCURLY      : "{"   ;
RCURLY      : "}"   ;
LPAREN      : "("   ;
RPAREN      : ")"   ;
POUND       : "#"   ;
QUESTION    : "?"   ;
SEMI        : ";"   ;
PLUS        : "+"   ;
LNOT        : "!"   ;
BNOT        : "~"   ;
BAND        : "&"   ;
RNAND       : "~&"  ;
BOR         : "|"   ;
RNOR        : "~|"  ;
BXOR        : "^"   ;
RXNOR       : "~^" | "^~" ;
STAR        : "*"   ;
DIV         : "/"   ;
MOD         : "%"   ;
EQUAL       : "=="  ;
NOT_EQ      : "!="  ;
NOT_EQ_CASE : "!==" ;
EQ_CASE     : "===" ;
LAND        : "&&"  ;
LOR         : "||"  ;
LT_         : "<"   ;
LE          : "<="  ;
GT          : ">"   ;
GE          : ">="  ;
SR          : ">>"  ;
SL          : "<<"  ;
TRIGGER     : "->"  ;
PPATH       : "=>"  ;
FPATH       : "*>"  ;

    // an identifier.  Note that testLiterals is set to true!  This means
    // that after we match the rule, we look in the Literals table to see
    // if it's a literal or really an identifer.
IDENTIFIER
      options {testLiterals=true;}
    :
        ('a'..'z'|'A'..'Z') ('a'..'z'|'A'..'Z'|'_'|'$'|'0'..'9')*
        ;
//"[A-Za-z]({_}[A-Za-z0-9]*)*"

ESCAPED_IDENTIFIER :
        '\\'! (~ '\040')+ ('\040'|'\t'|'\n')!
        ;

SYSTEM_TASK_NAME :
        '$' IDENTIFIER
        ;

    // string literals
STRING :
        '"' (~('"'|'\n'))* '"'
        ;

    // "compiler" define/macro.
DEFINE
      options {testLiterals=true;}
    :
    '`' IDENTIFIER
    ;

   // a dummy rule to force vocabulary to be all characters (except special
   //   ones that ANTLR uses internally (0 to 2)
protected
VOCAB :
        '\3'..'\177'
        ;

   // a numeric literal
//NUMBER :
//  ( (SIZE)? BASE SIZED_DIGIT ) => SIZED_NUMBER |
//  UNSIZED_NUMBER
//  ;

protected
SIZED_NUMBER :
    (SIZE)? BASE SIZED_DIGIT (SIZED_DIGIT | '_')*
    ;

protected
SIZE :
    (DIGIT)+
    ;

protected
BASE :
    '\'' ( 'd' | 'D' | 'h' | 'H' | 'o' | 'O' | 'b' | 'B' )
    ;

protected
SIZED_DIGIT :
    DIGIT | HEXDIGIT | 'x' | 'X' | 'z' | 'Z' | '?'
    ;

protected
UNSIZED_NUMBER :
    DIGIT (DIGIT | '_')* ( '.' (DIGIT | '_')* )? (EXPONENT)?
        ;

protected
DIGIT :
        ('0'..'9')
        ;

protected
HEXDIGIT :
        ('A'..'F'|'a'..'f')
        ;

protected
EXPONENT :
        ('e'|'E') ('+'|'-')? ('0'..'9')+
        ;



      
